#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Calculate Point and Figure Trends Based on Price Action
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
    [Description("Calculate Point and Figure Trends Based on Price Action\n\nwww.FreeIndicators.com")]
    public class PointAndFigurePrice : Indicator
    {
        #region Variables
        // Wizard generated variables
        // User defined variables (add any user defined variables below)
		private int signal = 10;        
		private string alertFile = @"Alert1.wav";
		private bool useHighLow = true;
		private bool colorBars = true;
		private bool audioAlerts = true;
		private bool drawArrows = true;
		
		private int trend = 0;
		private double hw = 0;						
        
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Add(new Plot(Color.Red, "Bullish Reversal"));
			Add(new Plot(Color.Blue, "Bearish Reversal"));
			
			Plots[0].Pen.Width=2;
			Plots[1].Pen.Width=2;
			
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			
            Name = "isPointAndFigurePrice";
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar<20)
				return;
			
			double h = useHighLow ? High[0] : Close[0];
			double l = useHighLow ? Low[0] : Close[0];
					
			// check for a change in trend and update highwater for current trend
			if (trend==1)
			{
				if (l<=hw)
				{
					trend=-1;
					hw=l+signal*TickSize;					
					if (drawArrows)
						DrawArrowDown(CurrentBar.ToString(),true,0,High[0]+2*TickSize,DownColor);
					if (audioAlerts)
						PlaySound(alertFile);					
				}
				else hw=Math.Max(hw,h-signal*TickSize);							
			}			
			else if (trend==-1)
			{
				if (h>=hw)
				{
					trend=1;
					hw=h-signal*TickSize;					
					if (drawArrows)
						DrawArrowUp(CurrentBar.ToString(),true,0,Low[0]-2*TickSize,UpColor);
					if (audioAlerts)
						PlaySound(alertFile);					
				}
				else hw=Math.Min(hw,l+signal*TickSize);												
			}					
			else
			{
				trend=1;
				hw=h-signal*TickSize;
			}
			
			// Color bars and Plot Reversal Lines
			if (trend==1)
			{
				if (colorBars)
					BarColor=UpColor;
				Bullish.Set(hw);
			}
			else if (trend==-1)
			{
				if (colorBars)
					BarColor=DownColor;
				Bearish.Set(hw);
			}
			else if (colorBars)
				BarColor=Color.Empty;			
        }

        #region Properties
		
		private Color UpColor
		{
			get { return Plots[0].Pen.Color; }
		}
		
		private Color DownColor
		{
			get { return Plots[1].Pen.Color; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Bullish
		{
			get { return Values[0]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Bearish
		{
			get { return Values[1]; }
		}
        
		[Description("Determine box size (in ticks) for specified indicator")]
		[Category("Parameters")]
		public int BoxSize
		{
			get { return signal; }
			set { signal = value; }
		}
		
		[Description("")]
		[Category("Parameters")]
		public string AlertFile
		{
			get { return alertFile; }
			set { alertFile = value; }
		}
		
		[Description("")]
		[Category("Parameters")]
		public bool UseHighLow
		{
			get { return useHighLow; }
			set { useHighLow = value; }
		}
		
		[Description("")]
		[Category("Parameters")]
		public bool ColorBars
		{
			get { return colorBars; }
			set { colorBars = value; }
		}
		
		[Description("")]
		[Category("Parameters")]
		public bool AudioAlerts
		{
			get { return audioAlerts; }
			set { audioAlerts = value; }
		}
		
		[Description("")]
		[Category("Parameters")]
		public bool DrawArrows
		{
			get { return drawArrows; }
			set { drawArrows = value; }
		}
		
		#endregion				
    }
}
#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private PointAndFigurePrice[] cachePointAndFigurePrice = null;

        private static PointAndFigurePrice checkPointAndFigurePrice = new PointAndFigurePrice();

        /// <summary>
        /// Calculate Point and Figure Trends Based on Price Action\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        public PointAndFigurePrice PointAndFigurePrice(string alertFile, bool audioAlerts, int boxSize, bool colorBars, bool drawArrows, bool useHighLow)
        {
            return PointAndFigurePrice(Input, alertFile, audioAlerts, boxSize, colorBars, drawArrows, useHighLow);
        }

        /// <summary>
        /// Calculate Point and Figure Trends Based on Price Action\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        public PointAndFigurePrice PointAndFigurePrice(Data.IDataSeries input, string alertFile, bool audioAlerts, int boxSize, bool colorBars, bool drawArrows, bool useHighLow)
        {
            if (cachePointAndFigurePrice != null)
                for (int idx = 0; idx < cachePointAndFigurePrice.Length; idx++)
                    if (cachePointAndFigurePrice[idx].AlertFile == alertFile && cachePointAndFigurePrice[idx].AudioAlerts == audioAlerts && cachePointAndFigurePrice[idx].BoxSize == boxSize && cachePointAndFigurePrice[idx].ColorBars == colorBars && cachePointAndFigurePrice[idx].DrawArrows == drawArrows && cachePointAndFigurePrice[idx].UseHighLow == useHighLow && cachePointAndFigurePrice[idx].EqualsInput(input))
                        return cachePointAndFigurePrice[idx];

            lock (checkPointAndFigurePrice)
            {
                checkPointAndFigurePrice.AlertFile = alertFile;
                alertFile = checkPointAndFigurePrice.AlertFile;
                checkPointAndFigurePrice.AudioAlerts = audioAlerts;
                audioAlerts = checkPointAndFigurePrice.AudioAlerts;
                checkPointAndFigurePrice.BoxSize = boxSize;
                boxSize = checkPointAndFigurePrice.BoxSize;
                checkPointAndFigurePrice.ColorBars = colorBars;
                colorBars = checkPointAndFigurePrice.ColorBars;
                checkPointAndFigurePrice.DrawArrows = drawArrows;
                drawArrows = checkPointAndFigurePrice.DrawArrows;
                checkPointAndFigurePrice.UseHighLow = useHighLow;
                useHighLow = checkPointAndFigurePrice.UseHighLow;

                if (cachePointAndFigurePrice != null)
                    for (int idx = 0; idx < cachePointAndFigurePrice.Length; idx++)
                        if (cachePointAndFigurePrice[idx].AlertFile == alertFile && cachePointAndFigurePrice[idx].AudioAlerts == audioAlerts && cachePointAndFigurePrice[idx].BoxSize == boxSize && cachePointAndFigurePrice[idx].ColorBars == colorBars && cachePointAndFigurePrice[idx].DrawArrows == drawArrows && cachePointAndFigurePrice[idx].UseHighLow == useHighLow && cachePointAndFigurePrice[idx].EqualsInput(input))
                            return cachePointAndFigurePrice[idx];

                PointAndFigurePrice indicator = new PointAndFigurePrice();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.AlertFile = alertFile;
                indicator.AudioAlerts = audioAlerts;
                indicator.BoxSize = boxSize;
                indicator.ColorBars = colorBars;
                indicator.DrawArrows = drawArrows;
                indicator.UseHighLow = useHighLow;
                Indicators.Add(indicator);
                indicator.SetUp();

                PointAndFigurePrice[] tmp = new PointAndFigurePrice[cachePointAndFigurePrice == null ? 1 : cachePointAndFigurePrice.Length + 1];
                if (cachePointAndFigurePrice != null)
                    cachePointAndFigurePrice.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cachePointAndFigurePrice = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Calculate Point and Figure Trends Based on Price Action\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PointAndFigurePrice PointAndFigurePrice(string alertFile, bool audioAlerts, int boxSize, bool colorBars, bool drawArrows, bool useHighLow)
        {
            return _indicator.PointAndFigurePrice(Input, alertFile, audioAlerts, boxSize, colorBars, drawArrows, useHighLow);
        }

        /// <summary>
        /// Calculate Point and Figure Trends Based on Price Action\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        public Indicator.PointAndFigurePrice PointAndFigurePrice(Data.IDataSeries input, string alertFile, bool audioAlerts, int boxSize, bool colorBars, bool drawArrows, bool useHighLow)
        {
            return _indicator.PointAndFigurePrice(input, alertFile, audioAlerts, boxSize, colorBars, drawArrows, useHighLow);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Calculate Point and Figure Trends Based on Price Action\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PointAndFigurePrice PointAndFigurePrice(string alertFile, bool audioAlerts, int boxSize, bool colorBars, bool drawArrows, bool useHighLow)
        {
            return _indicator.PointAndFigurePrice(Input, alertFile, audioAlerts, boxSize, colorBars, drawArrows, useHighLow);
        }

        /// <summary>
        /// Calculate Point and Figure Trends Based on Price Action\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        public Indicator.PointAndFigurePrice PointAndFigurePrice(Data.IDataSeries input, string alertFile, bool audioAlerts, int boxSize, bool colorBars, bool drawArrows, bool useHighLow)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.PointAndFigurePrice(input, alertFile, audioAlerts, boxSize, colorBars, drawArrows, useHighLow);
        }
    }
}
#endregion
